#!/usr/bin/perl -w 
#
#   grue, a bot that interfaces with frotz, allowing you to play infocom games on irc
#   Copyright (C) 2004 Robert Lorentz
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#   
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
my $VERSION = 0.41; # may 10, 2004

use POE;
use POE::Session;
use POE::Component::IRC;

use IO::Handle;		# autoflush

use Time::HiRes qw(time); # give greater precision to time()
use File::Temp qw(tempfile); 

use sigtrap 'handler' => \&quit_game, 'normal-signals'; # handle SIGINT, SIGTERM, SIGHUP

$| = 1; # for flushing stuff immediately

# create irc alias to use 
POE::Component::IRC->new('irc')
	|| die "Error creating irc alias: $!";


# thanks to Alex 'supersniffer' Hidalgo for helping me figure out irc clients generally accept 0x03 as a color code init code
my $colorcodeinit = 0x03;
# this is the list of colors grue can use.  these are NOT preserved between irc clients, afaik the colors just aren't standard.  
# Therefore it is an inevitable but that 'orange' may appear orange to one person, and brown to another.  Not really a bug
my $blue = sprintf("%c%c", $colorcodeinit, 0x32); 	# blue = '2'
my $green = sprintf("%c%c", $colorcodeinit, 0x33); 	# green = '3'
my $red = sprintf("%c%c", $colorcodeinit, 0x34);	# red = '4'
my $darkred = sprintf("%c%c", $colorcodeinit, 0x35);	# darkred = '5'
my $purple = sprintf("%c%c", $colorcodeinit, 0x36);	# purple = '6'
my $orange = sprintf("%c%c", $colorcodeinit, 0x37);	# orange = '7'
my $yellow = sprintf("%c%c", $colorcodeinit, 0x38);	# yellow = '8'
my $lightgreen = sprintf("%c%c", $colorcodeinit, 0x39);#lightgreen = '9'
my $teal = sprintf("%c%c%c", $colorcodeinit, 0x31, 0x30);#teal = '10'
my $aqua = sprintf("%c%c%c", $colorcodeinit, 0x31, 0x31);#aqua = '11'
my $brightblue = sprintf("%c%c%c", $colorcodeinit, 0x31, 0x32); # bright blue = '12'
my $brightpurple = sprintf("%c%c%c", $colorcodeinit, 0x31, 0x33);# bright purple = '13'
my $darkgrey = sprintf("%c%c%c", $colorcodeinit, 0x31, 0x34); # dark grey = '14'
my $grey = sprintf("%c%c%c", $colorcodeinit, 0x31, 0x35);	# grey = '15'
my $lightgray = sprintf("%c%c%c", $colorcodeinit, 0x31, 0x36); # light gray = '16'
my $nocolor = "";


#####################################
# user configuration begins here
#####################################


# this is our debug level.  if debug is 0 we get no messages (silent),
# if debug is 1 we get basic messages (connected, disconnected)
# if debug is 2 we get specific messages (directories created, games opened/closed)
# if debug is 3 we get all debug information
# by default debug is set to 1
my $debug = 1;

my $allowquit = 0; # allow irc users to make your bot quit?

# TODO allow for backup irc servers and ports?
my $ircserver = "irc.somenetwork.com";
my $ircport = '6667';
my $chan = '#grue';  # channel for bot to reside in
my @nick = qw(grue grau groo grew grah ghoo eurg); # list as many nicknames as you'd like, it will use first available
my $username = 'grue'; # nick!username@host
my $ircname = "It is very dark...  grue v$VERSION";
my $irclinemax = 255; # we won't send lines longer than this (really this -1 -$textcolorlength)

my $ircreconnecttime = 30; # 30 second irc reconnect timeout 
my $pingdelay = 300; # ping delay in seconds.  This ONLY happens if there is no activity

my $textcolor = $nocolor; # by default we will use no textcolor...

my $homedir = $ENV{"PWD"}
	|| die "You have not set your PWD environment variable, something weird is going on";
my $dumbfrotz = "$homedir/dumb-frotz"; # you need a special 'flushing' dumb-frotz, give the path to it here
my $gamesdir = "$homedir/infocomgames/"; # this is where all the infocom games go (subdirectories ignored)
my $savesdir = "$homedir/infocomsaves/"; # for every game, if there are save files then they will be under 
				         # $savesdir/gamename/ to keep separation/organization easier

my $prefix = ','; # by default all commands start with ,, and game actions begin with ,
	          # to change this to a period or anything else (even multiple character) just change this.

# this is how long of a delay we have before checking for output from the game.  this
# can probably be 1 second or more without becoming apparant (on slower machines this
# would be smart)
my $ticklength = 0.5;

my $changetopic = 1;  # for the 'title' stuff, grue is set up by default to change the topic.  If he doesn't have ops, or channel is not -t, then he can't do this.. so you should set this to 0.  Or if you just don't want him to, you should set this to 0.
#TODO TODO TODO make it so he detects if he can change the topic, and if not have him just say it


# comment out this next line by removing it or adding a # before it, or else this bot will not work.
# this is to ensure that you actually configure this bot
die "I have not looked at the config well enough.  I should be ashamed!!";

#####################################
# user configuration ends here
#####################################

my $whichnick = 0; # which nickname did we end up using?

my $buffer = ""; # this is a buffer for our output, so we don't scroll lines so hard..
my $tmp = "";    # also for our output

my $connected = 0;
my $playing = ""; # are we playing a game?  if so, what is the name?

my $timestarted = 0;

(undef, $tmpfile) = tempfile(); # get our /tmp/ file
if($debug == 3)
{
	print "using \$tmpfile of: $tmpfile\n";
}

my $disconnect = 0; # are we disconnected?


POE::Session->new ( _start     => \&irc_start,
                    irc_376    => \&irc_connect,
		    irc_433    => \&irc_nicktaken,
                    irc_public => \&irc_pub_msg,
	            irc_disconnected => \&irc_disconnected,
		    irc_error => \&irc_disconnected,
		    irc_socketerr => \&irc_disconnected,
	    	    autoping => \&autoping);


sub irc_nicktaken
{
	if($debug >= 1)
	{
		print "Nick $nick[$whichnick] taken ";
	}
	$whichnick++;
	if($whichnick == @nick) # this means we've gotten to the end of the list..
	{
		$whichnick = 0;
	}
	
	$_[KERNEL]->post('irc' => 'nick' => "$nick[$whichnick]");
	
	if($debug >= 1)
	{
		print "trying  $nick[$whichnick]... ";
	}
}

# thanks again POE cookbook :)
sub autoping
{
	if(!$connected)
	{
		return;
	}
	
	my $kernel = $_[KERNEL];
	my $heap = $_[HEAP];

	$kernel->post('irc' => 'userhost' => "$nick[$whichnick]")
		unless $heap->{seen_traffic};
		
	$heap->{seen_traffic} = 0;
	$kernel->delay('autoping' =>  $pingdelay);
}
	    
sub quit_game
{
	close_game();
	$disconnect = 1;
	
	if($connected == 1)
	{
		$poe_kernel->post('irc', 'quit', "quit was issued");
		if($debug == 3)
		{
			print "quit was issued\n";
		}
	}
}
	
# this lets us load up a game!
sub load_game
{
	my $game = $_[0];
	chomp($game);

	if (!$connected) # how are we even here!
	{
		return;
	}

	if($playing ne "")
	{
		$poe_kernel->post('irc', 'privmsg', $chan, "Cannot load game $game, already playing $playing!  Try ${prefix}${prefix}command close");
		return;
	}
	
	if(!(-e $game)) # if the game doesn't exist we have a problem!
	{
		$poe_kernel->post('irc', 'privmsg', $chan, "$game does not exist, please try again!");
		return;
	}
	
	if(!(-r $game)) # if we can't read it, this is also a problem!
	{
		$poe_kernel->post('irc', 'privmsg', $chan, "$game is not readable by this user, please try again!");
		return;
	}
	
	$poe_kernel->post('irc', 'privmsg', $chan, "Attemping to load game $game.." ); # 
	
	if(open(PROC, "| $dumbfrotz $game >$tmpfile"))
	{
		PROC->autoflush(); # autoflushing is required..
		
		if($debug == 2)
		{
			print "Successfully loaded $game!\n";
		}
		$poe_kernel->post('irc', 'privmsg', $chan, "Successfully loaded $game!");
		$playing = $game;
	
		open(IN, $tmpfile)
			|| die "Cannot open $tmp for input!"; # open up our input	

		$timestarted = time(); # interesting stats 
	}
	else
	{
		if($debug == 2)
		{
			print "Problem loading $game: $!\n";
		}
		$poe_kernel->post('irc', 'privmsg', $chan, "Problem loading $game: $!");
	}
}

# 'if you do something more than once...'
sub time_played
{
	my $tmpmin = 0;
	my $tmpsec = 0;
	my $tmp = 0;
	
	$tmp = time() - $timestarted;
	
	$tmpmin += int($tmp / 60);
	$tmp -= $tmpmin * 60; # 60 secs/minute

	$tmpsec = int($tmp); # grab seconds, but don't mess with decimals
	
	return (sprintf "%2d:%02d", $tmpmin, $tmpsec);
}


sub close_game
{
	if($playing eq "") # some idiot tried to close when we had nothing open
	{
		return;
	}
	
	close(IN)
		|| warn "OOPS could not close filehandle: $!"; # close our output
	close(PROC)
		|| warn "OOPS could not close proc: $!"; # close other process
		
	
	if($debug >= 2)
	{
		print "Successfully closed game $playing, played for " . time_played() . "\n";
	}
	$poe_kernel->post('irc', 'privmsg', $chan, "Successfully closed game $playing, played for " . time_played());
	
	$timestarted = 0;
	$playing = "";

	if($changetopic == 1)
	{
		$poe_kernel->post('irc', 'topic', $chan, "$nick[$whichnick], at your service!  try ${prefix}${prefix}help for commands" ); 
	}
	else
	{
		$poe_kernel->post('irc', 'privmsg', $chan, "$nick[$whichnick], at your service!  try ${prefix}${prefix}help for commands" );
	}
}
	    
# this is called at the beginning of everything, so we set up our irc connection etc
sub irc_start 
{
	$_[KERNEL]->post('irc', 'register', 'all');
	if($debug >= 1)
	{
		print "Connecting to irc... ";
	}

	# connect to irc!
	$_[KERNEL]->post('irc', 'connect',    {
		Server        => $ircserver,
		Nick          => $nick[$whichnick],
		Port          => $ircport,
		Username      => $username,
		Ircname       => $ircname } );
}

# this event is handled when we actually connect to irc
sub irc_connect 
{
	# ping stuff
	$_[HEAP]->{seen_traffic} = 1;
	$_[KERNEL]->delay('autoping' => $pingdelay);
	
	if($debug >= 1)
	{
		print "ok!\n"; # connected successfully!
		print "Joining $chan... ";
	}

	$_[KERNEL]->post('irc' => 'join' => $chan);
	if($debug >= 1)
	{
		print "ok!\n";
	}
	if($changetopic == 1)
	{
		$poe_kernel->post('irc', 'topic', $chan, "$nick[$whichnick], at your service!  try ${prefix}${prefix}help for commands" ); 
	}
	else
	{
		$poe_kernel->post('irc', 'privmsg', $chan,  "$nick[$whichnick], at your service!  try ${prefix}${prefix}help for commands" );
	}
	
	$connected = 1;
}

sub output_files
{
	my @files = @_;

	my $counter = 0;
	my $output = "";
			
	foreach (@files)
	{
		chomp($_);
		
		$counter++;
			
		$output = $output . $_ . "   ";
				
		if($counter == 5) # 5 files listed per row 
		{
			$counter = 0;
			$poe_kernel->post('irc', 'privmsg', $chan, $output);
			$output = ""; # clear output
		}
	}

	if($counter > 0)
	{
		$poe_kernel->post('irc', 'privmsg', $chan, $output);
	}
}

# prevent users from stealing my megahurts
sub unhax 
{
	my $result = $_[0];
	
	$result =~ s/^\.+//; # take out any preceding .'s
	$result =~ s/\/.*//g; # take out any /'s (followed by .'s, as a filename of /.. would turn into ..
	$result =~ s/\|.*//g; # also take out the | character, and any .'s that may follow
	$result =~ s/`.*//g; # and the `

	return $result;
}

# i am putting in my own save and restore functions for 2 reasons
# 1) the way everything works out, saving is 'blind'.. you have to issue a save command,
#    tell it where to save, and then say 'y' if you want to overwrite - all without seeing
#    anything back from the game.  It's an issue with how my linewrapping works.
# 2) i'd have to mess with frotz itself to disable people from saving files anywhere they'd
#    like.  better secure than sloppy I guess!
sub save 
{
	my $exists = 0;
	
	my $playingdir = $playing;
	$playingdir =~ s/\/.*\///;
	$playingdir =~ s/\..*//;	# clear off directory path, and extension

	# if the directory doesn't exist, lets create it
	if(!(-e ($savesdir . $playingdir)))
	{
		mkdir($savesdir . $playingdir) 
			|| die "Cannot create directory" . $savesdir . $playingdir . "!";
		if($debug >= 2)
		{
			print "Created directory" . $savesdir . $playingdir . " sucecssfully!\n";
		}
	}
	
	# add the game subdir on top of this too
	if(-e ($savesdir . $playingdir . "/" . $_[0]))
	{
		$exists = 1;
	}
	
	print PROC "save\n"; # we want to save

	print PROC ($savesdir . $playingdir . "/" . $_[0] . "\n"); # save to savedir/filename

	if($exists == 1)
	{
		print PROC "y\n"; # yes, we want to overwrite
	}
}

sub restore
{
	my $playingdir = $playing;
	$playingdir =~ s/\/.*\///;
	$playingdir =~ s/\..*//;	# clear off directory path, and extension

	# if doesn't exist savesdir/subdir then exit (doesn't exist)
	if(!(-e $savesdir . $playingdir . "/" . $_[0]))
	{
		$poe_kernel->post('irc', 'privmsg', $chan, "Error!  Cannot restore, " . $savesdir . $playingdir . "/" . $_[0] . " does not exist.");
		return;
	}

	print PROC "restore\n";

	print PROC $savesdir . $playingdir . "/" . $_[0] . "\n";
}

sub setcolor
{
	my $outputstr = "";
	$_ = $_[0];

	if(/^blue/i)
	{
		$outputstr = "blue";
		$textcolor = $blue;
	}
	elsif(/^green/i)
	{
		$outputstr = "green";
		$textcolor = $green;
	}
	elsif(/^red/i)
	{
		$outputstr = "red";
		$textcolor = $red;
	}
	elsif(/^darkred/i)
	{
		$outputstr = "darkred";
		$textcolor = $darkred;
	}
	elsif(/^purple/i)
	{
		$outputstr = "purple";
		$textcolor = $purple;
	}
	elsif(/^orange/i)
	{
		$outputstr = "orange";
		$textcolor = $orange;
	}
	elsif(/^yellow/i)
	{
		$outputstr = "yellow";
		$textcolor = $yellow;
	}
	elsif(/^light\s*green/i)
	{
		$outputstr = "light green";
		$textcolor = $lightgreen;
	}
	elsif(/^teal/i)
	{
		$outputstr = "teal";
		$textcolor = $teal;
	}
	elsif(/^aqua/i)
	{
		$outputstr = "aqua";
		$textcolor = $aqua;
	}
	elsif(/^bright\s*blue/i)
	{
		$outputstr = "bright blue";
		$textcolor = $brightblue;
	}
	elsif(/^bright\s*purple/i)
	{
		$outputstr = "bright purple";
		$textcolor = $brightpurple;
	}
	elsif(/^dark\s*gr(?:a|e)y/i)
	{
		$outputstr = "dark grey";
		$textcolor = $darkgrey;
	}
	elsif(/^gr(?:a|e)y/i)
	{
		$outputstr = "grey";
		$textcolor = $grey;
	}
	elsif(/^light\s*gr(?:e|a)y/i)
	{
		$outputstr = "light grey";
		$textcolor = $lightgrey;
	}
	elsif(/^(?:none|reset|blank)/i)
	{
		$outputstr = "none";
		$textcolor = $nocolor;
	}
	else
	{
		$poe_kernel->post('irc', 'privmsg', $chan, "Error, invalid color $_!  Try ${prefix}${prefix}color for list of available colors!");
		return;
	}

	
	$poe_kernel->post('irc', 'privmsg', $chan, $textcolor . "Color set to $outputstr");

	return;
}
		
sub irc_disconnected
{
	$connected = 0;
	
	# this function gets called if we have an irc connection issue, or if we explicitly state
	# to disconnect.  therefore we distinguish between the two with $disconnect
	if($disconnect)
	{
		unlink($tmpfile)
			|| warn "OOPS could not remove $tmpfile  $!"; # remove the temporary file
		
			die "Exiting on user request/irc disconnected\n";
	}
	else
	{
		if($debug >= 1)
		{
			print "Disconnected!  Trying to reconnect.. \n";
		}
		sleep($ircreconnecttime);
		# connect to irc!
		$_[KERNEL]->post('irc', 'connect',    {
			Server        => $ircserver,
			Nick          => $nick[$whichnick],
			Port          => $ircport,
			Username      => $username,
			Ircname       => $ircname } );
	}
		
}

sub irc_pub_msg
{
	$_ = $_[ARG2];
	$_[HEAP]->{seen_traffic} = 1;
	
	my @files;
	
	my $counter;
	my $output;

	if(/^(${prefix}${prefix}\S+)/i) # ,,<something>
	{
		s/^${prefix}${prefix}//; # remove the leading ,,'s
		
		if(/^quit/i)
		{
			if($allowquit == 1)
			{
				quit_game(); # quit out
			}
			else
			{
				$poe_kernel->post('irc', 'privmsg', $chan, "This command has been disabled by the bot owner.");
			}
		}
		elsif(/(?:^(open|load))\s+\S+/i) # load a game!
		{
			s/^$1 +//i; # get the filename
			
			$_ = unhax($_); # format this potentially nasty user input
			
			$_ = $gamesdir . $_ ;
			
			load_game($_);
		}
		elsif(/^close*/i) # close current game
		{
			close_game();
		}
		elsif(/^game*/i) # games
		{
			$poe_kernel->post('irc', 'privmsg', $chan, "Games available are as follows... (for a synopsis of all games here, visit http://www.latz.org/games/list.shtml)"); 
			output_files(`ls $gamesdir`);
		}
		elsif(/^save*/i) #saves
		{
			if($playing eq "")
			{
				$poe_kernel->post('irc', 'privmsg', $chan, "You must begin playing a game before checking available saves!");
			}
			else
			{
				my $playingdir = $playing;	# temporary storage
				$playingdir =~ s/\/.*\///;
				$playingdir =~ s/\..*//;	# clear off directory path, and extension
				
				if(-e $savesdir . $playingdir)
				{
					$poe_kernel->post('irc', 'privmsg', $chan, "Saves available for " . $playingdir . " are..."); 
					$playingdir = $savesdir . $playingdir; # doing `ls $savesdir . $playingdir` doesn't work out quite right
					output_files(`ls $playingdir`);
				}
				else
				{
					$poe_kernel->post('irc', 'privmsg', $chan, "No saves available for " . $playingdir . "!"); 
				}
			}
		}
		elsif(/^help*/i)
		{
			$poe_kernel->post('irc', 'privmsg', $chan, "Preface all game actions with a ${prefix} (i.e.: ${prefix}look)  Preface all bot actions with ${prefix}${prefix} (i.e. ${prefix}${prefix}open game.dat)");
			$poe_kernel->post('irc', 'privmsg', $chan, "${prefix}${prefix} commands are as follows: open <datafile> (opens specified game)  close (closes open game)  color <textcolor>");
			$poe_kernel->post('irc', 'privmsg', $chan, "                            games (lists games)  saves (lists saves) quit (shut down bot) about (general information)");
		}
		elsif(/^color(?: +(\S+)|.*)/i)  
		{
			if(!defined($1))
			{
				$poe_kernel->post('irc', 'privmsg', $chan, "Use ,,color <colorname>   Valid colors are...");
				$poe_kernel->post('irc', 'privmsg', $chan, $blue . "blue " . $green . "green " . $red . "red " . $darkred . "darkred " . $purple . "purple " . $orange . "orange " . $yellow . "yellow " . $lightgreen . "lightgreen " . $teal . "teal " . $aqua . "aqua ");
				$poe_kernel->post('irc', 'privmsg', $chan, $brightblue . "brightblue " . $brightpurple . "brightpurple " . $darkgrey . "darkgrey " . $grey . "grey " . $lightgrey . "lightgrey " . $nocolor . "none");
			}
			else
			{
				setcolor($1);
			}
		}	
			
		elsif(/^about*/i)
		{
			$poe_kernel->post('irc', 'privmsg', $chan, "I am grue v$VERSION, an irc bot that lets you play Infocom games!  My author is Robert Lorentz (rcl\@lucidsoft.org) and I am available at http://grue.sf.net");
			$poe_kernel->post('irc', 'privmsg', $chan, "${prefix}${prefix} help lists my commands.  For information on Infocom games I suggest http://infocom.elsewhere.org/ and http://www.latz.org/games/list.shtml");
		}

		else # confused eh?
		{
			$poe_kernel->post('irc', 'privmsg', $chan, "Invalid command, try ${prefix}${prefix}help");
		}
	}
	elsif((/^${prefix}/) && ($playing ne "")) # the message starts with a , which means it is important to us.  don't do any of this if not playing a game
        {
	        s/^${prefix}\s*//; # get the meat of the command 
		
		if(/^save\s+(\S+)/i) # user tried to save, let's do this ourselves
		{
			$_ = unhax($1);
			save($_);
		}
		elsif(/^save/i) 
		{
			$poe_kernel->post('irc', 'privmsg', $chan, "Please use ${prefix}save in the format: ${prefix}save <savefile>");
		}
		elsif(/^restore\s+(\S+)/i) # restore routine
		{
			$_ = unhax($1);
			restore($_);
		}
		elsif(/^restore/i)
		{
			$poe_kernel->post('irc', 'privmsg', $chan, "Please use ${prefix}restore in the format: ${prefix}restore <savefile>");
		}
		elsif(/^quit/i)
		{
			close_game(); # instead of having the game handle this, we'll just do it ourselves
		}
		else # we didn't intercept a command, pass as normal
		{	
			print PROC $_ . "\n";
		}
	}
	# otherwise it was IDLE BANTER
}


# this session is here to keep track of our timer callback to check and output stuff
# thanks #perl on freenode.net and the POE Cookbook :)
POE::Session->create
(	inline_states =>
	{
		_start => sub {
			$_[HEAP]->{next_alarm_time} = int( time() ) + 3; # 3 second delay on the first one
			$_[KERNEL]->alarm( tick => $_[HEAP]->{next_alarm_time} );
		},

		tick => sub {
		
			if($playing eq "")
			{
				# preserve our timer!!
	                        $_[HEAP]->{next_alarm_time} += $ticklength; # $ticklength seconds until next tick
	                        $_[KERNEL]->alarm( tick => $_[HEAP]->{next_alarm_time} );
											
				return;
			}
			else
			{
				while(<IN>) 
				{
					$tmp = $_;

					if(/\[Press RETURN or ENTER/) # this happens in the intro of plundered hearts (don't ask why I played this game)
					{
						print PROC " \n";
					}
					if(/\(Hit RETURN or ENTER when ready\.\)/) # hitchhiker's guide
					{
						print PROC " \n";
					}
					if(/RETURN or ENTER/) # banking on this never occurring except when the game wants us to do it to continue :) (generic)
					{
						print PROC " \n";
					}
					if(/\[Press any key to begin\.\]/) # nord and bert
					{
						print PROC " \n";
					}

					if(/(?:( *(Score: [0-9]*) *(Moves: [0-9]*)))|(?:^(.*Time:.*[0-9]:[0-9][0-9]\s+\S+\s+))/) 
					# this is Definitely the title, so lets set it 
					# first part is for Most games, 
					# 2nd part is for Moon Mist
					{
						s/^\s*//; # because of formatting it seems the locations are ofset by spacing.  This is pointless in the topic

						# before we do anything, we need to look at the beginning of this and see if it has a preceding ' > '
						# if things are done very quickly, this tends to pop up into the topic.  I think this has to do
						# with things outracing my timer, so this is a smarter fix than making the timer cycle faster
						s/^\s*>\s*//;
						
						s/\s{5,}/     /g; # lets kill some excess spaces
						$tmp = $_;
						chomp($tmp);

						$_ = $playing;
						s/\/.*\///; # only display the filename, not path
						s/\.dat//i; # furthermore, take the .dat out of this
						s/_/ /;     # while i am on a roll, replace _ with ' ' :)
					
						$tmp = $tmp . "   [" . time_played() . "]  (" . $_ . ")";
					
						if($changetopic == 1)
						{
							$_[KERNEL]->post('irc', 'topic', $chan, $tmp);
						}
						else
						{
							$_[KERNEL]->post('irc', 'privmsg', $chan, $tmp);
						}
					}
					elsif(/^>/) # this means we've reached an end of message, so output!
					{
						if(length $buffer > ($irclinemax - (length $textcolor))) # $irclinemax characters a line in irc is max 
						{
							$digit = 0;
						
							while(($digit <= (length $buffer)) && (defined($_ = substr($buffer, $digit, ($irclinemax - (length $textcolor))))))
							{
								if(length $_ >= ($irclinemax - (length $textcolor)))
								{
									s/(\s\S*)$//; # we do this for line wrapping (THANKS BALDURK)
									$digit -= length $1 # however long we chopped off, lets subtract this from digit
								}
						
								if($_ ne "") # only output if there is actually something here
								{
									s/\s*$//; # don't output trailing spaces, its just a waste of bandwidth
									$_ = $textcolor . $_;
									$_[KERNEL]->post('irc', 'privmsg', $chan, $_); # output to irc!
								}
								$digit += $irclinemax - (length $textcolor); 
							}
						}
						else
						{
							$buffer =~ s/\s*$//; # don't output trailing spaces, its just a waste of bandwidth
							if($buffer ne "") # only output if there is actually something here
							{
								$buffer = $textcolor . $buffer;
								$_[KERNEL]->post('irc', 'privmsg', $chan, $buffer); # output to irc!
							}
						}
						$buffer = "";
					}
					else # if these two aren't the case, just add on to $buffer
					{
						$buffer = $buffer . " " . $tmp;
						chomp($buffer);
					}
				}
			
				$_[HEAP]->{next_alarm_time} += $ticklength; # $ticklength seconds until next tick
				$_[KERNEL]->alarm( tick => $_[HEAP]->{next_alarm_time} );
			} 
		}
	}
);
				
$poe_kernel->run();		 

